// -*- Mode: Go; indent-tabs-mode: t -*-

/*
 * Copyright (C) 2015 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package runner

import (
	"fmt"
	"log"
	"strings"

	"launchpad.net/snappy-tests-job/utils"
)

const (
	goPkg            = "src/launchpad.net/snappy"
	goDepsInstallCmd = "go get launchpad.net/godeps"
	depsInstallCmd   = "godeps -u dependencies.tsv"
)

// Runner is the test runner type
type Runner interface {
	RunTests(path, cmdTpl, ip string, fromBranch bool) error
}

// BasicRunnerHandler is a basic type implementing Runner
type BasicRunnerHandler struct {
	util utils.Utilizer
}

// NewBasicRunnerHandler is the BasicRunnerHandler constructor
func NewBasicRunnerHandler(util utils.Utilizer) *BasicRunnerHandler {
	return &BasicRunnerHandler{util: util}
}

// RunTests executes the given command in the given path. The command is formed
// by interpolating the ip in the cmdTpl; before it installs godeps and the
// required dependencies
func (r *BasicRunnerHandler) RunTests(path, cmdTpl, ip string, fromBranch bool) (err error) {
	goPath := determineGoPath(path)

	log.Print("*** Installing godeps ***")
	err = r.installGodeps(path, goPath)

	if err != nil {
		return
	}

	log.Print("*** Installing dependencies ***")
	err = r.installDeps(path, goPath)

	if err != nil {
		return
	}

	return r.runTests(path, goPath, cmdTpl, ip, fromBranch)
}

func (r *BasicRunnerHandler) runTests(path, goPath, cmdTpl, ip string, fromBranch bool) (err error) {
	var snappyFromBranch string
	if fromBranch {
		snappyFromBranch = "-snappy-from-branch"
	}

	cmd := fmt.Sprintf(cmdTpl, ip, snappyFromBranch)

	log.Printf("*** Running tests with command %s in %s ***", cmd, path)

	params := utils.NewExecCommandParams(path, strings.Fields(cmd), true)

	params.AppendEnv(fmt.Sprintf("GOPATH=%s", goPath))

	_, err = r.util.ExecCommand(params)

	return
}

func (r *BasicRunnerHandler) installGodeps(path, goPath string) (err error) {
	params := utils.NewExecCommandParams(path, strings.Fields(goDepsInstallCmd), true)

	params.AppendEnv(fmt.Sprintf("GOPATH=%s", goPath))

	_, err = r.util.ExecCommand(params)

	return
}

func (r *BasicRunnerHandler) installDeps(path, goPath string) (err error) {
	cmd := goPath + "/bin/" + depsInstallCmd
	params := utils.NewExecCommandParams(path, strings.Fields(cmd), true)

	params.AppendEnv(fmt.Sprintf("GOPATH=%s", goPath))

	_, err = r.util.ExecCommand(params)

	return
}

func determineGoPath(path string) string {
	return strings.Replace(path, "/"+goPkg, "", 1)
}
