// -*- Mode: Go; indent-tabs-mode: t -*-

/*
 * Copyright (C) 2015 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package runner

import (
	"fmt"
	"path/filepath"
	"strings"
	"testing"

	check "gopkg.in/check.v1"

	"launchpad.net/snappy-tests-job/testhelpers"
	"launchpad.net/snappy-tests-job/utils"
)

var _ = check.Suite(&runnerSuite{})

type runnerSuite struct {
	fakeUtil   *testhelpers.FakeUtils
	subject    Runner
	ip         string
	path       string
	goPath     string
	cmdTpl     string
	fromBranch bool
}

func Test(t *testing.T) { check.TestingT(t) }

func (s *runnerSuite) SetUpTest(c *check.C) {
	s.fakeUtil = testhelpers.NewFakeUtils()

	s.subject = NewBasicRunnerHandler(s.fakeUtil)

	s.ip = "2.2.2.2"
	s.goPath = "/some/path/with/a/pakage/name/in/it"
	s.path = filepath.Join(s.goPath, goPkg)
	s.cmdTpl = "go run %s %s"
}

func (s *runnerSuite) TestNewRunner(c *check.C) {
	_, ok := s.subject.(Runner)
	c.Assert(ok, check.Equals, true,
		check.Commentf("Expected type Runner, got %T", s.subject))
}

func (s *runnerSuite) TestRunCallRunTestsCommand(c *check.C) {
	s.subject.RunTests(s.path, s.cmdTpl, s.ip, s.fromBranch)

	runCmd := runCmd(s.path, s.cmdTpl, s.ip, s.goPath, false)
	c.Assert(s.fakeUtil.ExecCalls[runCmd], check.Equals, 1,
		check.Commentf("The command %s was not called", runCmd))
}

func (s *runnerSuite) TestRunCallInstallGodepsCommand(c *check.C) {
	s.subject.RunTests(s.path, s.cmdTpl, s.ip, s.fromBranch)

	installGodepCmd := installGodepCmd(s.path, s.goPath)
	c.Assert(s.fakeUtil.ExecCalls[installGodepCmd], check.Equals, 1,
		check.Commentf("The command %s was not called", installGodepCmd))
}

func (s *runnerSuite) TestRunCallInstallDependenciesCommand(c *check.C) {
	s.subject.RunTests(s.path, s.cmdTpl, s.ip, s.fromBranch)

	installDepCmd := installDepCmd(s.path, s.goPath)
	c.Assert(s.fakeUtil.ExecCalls[installDepCmd], check.Equals, 1,
		check.Commentf("The command %s was not called", installDepCmd))
}

func (s *runnerSuite) TestRunReturnError(c *check.C) {
	s.fakeUtil.FailExec = true

	err := s.subject.RunTests(s.path, s.cmdTpl, s.ip, s.fromBranch)

	c.Assert(err, check.NotNil, check.Commentf("Expected error not returned by Runner.RunTests"))
}

func (s *runnerSuite) TestRunCallRunTestsFromBranchCommand(c *check.C) {
	fromBranch := true
	s.subject.RunTests(s.path, s.cmdTpl, s.ip, fromBranch)

	runCmd := runCmd(s.path, s.cmdTpl, s.ip, s.goPath, fromBranch)
	c.Assert(s.fakeUtil.ExecCalls[runCmd], check.Equals, 1,
		check.Commentf("The command %s was not called", runCmd))
}

func runCmd(path, cmdTpl, ip, goPath string, fromBranch bool) string {
	var snappyFromBranch string
	if fromBranch {
		snappyFromBranch = "-snappy-from-branch"
	}

	cmd := fmt.Sprintf(cmdTpl, ip, snappyFromBranch)
	params := utils.NewExecCommandParams(path, strings.Fields(cmd), true)
	params.AppendEnv(fmt.Sprintf("GOPATH=%s", goPath))
	return params.String()
}

func installGodepCmd(path, goPath string) string {
	cmd := goDepsInstallCmd
	params := utils.NewExecCommandParams(path, strings.Fields(cmd), true)
	params.AppendEnv(fmt.Sprintf("GOPATH=%s", goPath))
	return params.String()
}

func installDepCmd(path, goPath string) string {
	cmd := goPath + "/bin/" + depsInstallCmd
	params := utils.NewExecCommandParams(path, strings.Fields(cmd), true)
	params.AppendEnv(fmt.Sprintf("GOPATH=%s", goPath))
	return params.String()
}
