// -*- Mode: Go; indent-tabs-mode: t -*-

/*
 * Copyright (C) 2015 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package source

import (
	"os"
	"path/filepath"
	"strconv"
	"testing"

	check "gopkg.in/check.v1"

	"launchpad.net/snappy-tests-job/testhelpers"
	"launchpad.net/snappy-tests-job/utils"
)

const testRepo = "lp:snappy"

var _ = check.Suite(&sourceSuite{})

type sourceSuite struct {
	fakeUtil *testhelpers.FakeUtils
	subject  Sourcer
}

func Test(t *testing.T) { check.TestingT(t) }

func (s *sourceSuite) SetUpTest(c *check.C) {
	s.fakeUtil = testhelpers.NewFakeUtils()

	s.subject = NewBzrHandler(s.fakeUtil)
}

func (s *sourceSuite) TestNewBzrHandler(c *check.C) {
	_, ok := s.subject.(Sourcer)
	c.Assert(ok, check.Equals, true,
		check.Commentf("Expected type Sourcer, got %T", s.subject))
}

func (s *sourceSuite) TestRunCreatesTargetPath(c *check.C) {
	s.subject.Run(testRepo)

	expected := expectedBasePath()

	c.Assert(s.fakeUtil.MkDirCalls[expected], check.Equals, 1,
		check.Commentf("The target directory %s was not created", expected))
}

func (s *sourceSuite) TestRunCallsBranchCommand(c *check.C) {
	s.subject.Run(testRepo)

	pullCmd := pullCmd()
	c.Assert(s.fakeUtil.ExecCalls[pullCmd], check.Equals, 1,
		check.Commentf("The pull command %s was not called", pullCmd))
}

func (s *sourceSuite) TestRunReturnValue(c *check.C) {
	path, err := s.subject.Run(testRepo)
	expected := expectedReturnPath()

	c.Assert(path, check.Equals, expected, check.Commentf("Unexpected return value path %s", path))
	c.Assert(err, check.IsNil, check.Commentf("Unexpected return error %s", err))
}

func (s *sourceSuite) TestRunDoesNotCallBranchCommandOnError(c *check.C) {
	s.fakeUtil.FailMkDir = true

	s.subject.Run(testRepo)

	pullCmd := pullCmd()
	c.Assert(s.fakeUtil.ExecCalls[pullCmd], check.Equals, 0,
		check.Commentf(
			"Pull command called after failure creating target directory, called %d",
			s.fakeUtil.ExecCalls[pullCmd]))
}

func expectedBasePath() string {
	return filepath.Join(targetPath, strconv.Itoa(os.Getpid()), basePkg)
}

func expectedReturnPath() string {
	return filepath.Join(expectedBasePath(), targetSubdir)
}

func pullCmd() string {
	params := utils.NewExecCommandParams(expectedBasePath(), []string{"bzr", "branch", testRepo, targetSubdir}, true)
	return params.String()
}
