// -*- Mode: Go; indent-tabs-mode: t -*-

/*
 * Copyright (C) 2015 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package utils

import (
	"log"
	"os"
	"os/exec"
	"strings"
)

// Utilizer is the interface implemented by all the utility types
type Utilizer interface {
	PrepareTargetDir(string) error
	ExecCommand(*ExecCommandParams) (output string, err error)
}

// BasicHandler is a utility type
type BasicHandler struct {
}

// ExecCommandParams is the struct defining the ExecCommand parameters
type ExecCommandParams struct {
	path string
	cmds []string
	pipe bool
	env  []string
}

// NewBasicHandler is a constructor for BasicHandler
func NewBasicHandler() *BasicHandler {
	return &BasicHandler{}
}

// PrepareTargetDir creates the given target directory, removing it previously if it didn't exist
func (u *BasicHandler) PrepareTargetDir(targetDir string) error {
	if _, err := os.Stat(targetDir); err == nil {
		// dir exists, remove it
		os.RemoveAll(targetDir)
	}
	return os.MkdirAll(targetDir, 0777)
}

// ExecCommand executes the given command and pipes the results to os.Stdout and os.Stderr, returning the resulting error
func (u *BasicHandler) ExecCommand(params *ExecCommandParams) (output string, err error) {
	cmd := exec.Command(params.cmds[0], params.cmds[1:]...)

	if params.path != "" {
		cmd.Dir = params.path
	}

	if params.env != nil {
		env := os.Environ()
		cmd.Env = append(params.env, env...)
	}

	if params.pipe {
		cmd.Stdout = os.Stdout
		cmd.Stderr = os.Stderr
		err = cmd.Run()
	} else {
		var res []byte
		res, err = cmd.CombinedOutput()
		output = string(res)
	}

	if err != nil {
		log.Printf("Error while running %s: %s", cmd.Args, err)
	}
	return
}

// NewExecCommandParams is the ExecCommandParams constructor
func NewExecCommandParams(path string, cmds []string, pipe bool) *ExecCommandParams {
	return &ExecCommandParams{path: path, cmds: cmds, pipe: pipe}
}

// ExecCommandStr returns the output string describing the command being executed
func (params *ExecCommandParams) String() (output string) {
	if params != nil {
		var pipeStr string
		if !params.pipe {
			pipeStr = "out"
		}
		var env string
		if params.env != nil {
			env = " and env " + strings.Join(params.env, ", ")
		}
		output = strings.Join(append(
			params.cmds, "# executed in "+params.path+" with"+pipeStr+" pipe"+env), " ")
	}
	return
}

// AppendEnv adds one string to the env field
func (params *ExecCommandParams) AppendEnv(value string) {
	params.env = append([]string{value}, params.env...)
}
